-----------------------------------------------------------------------------------------------
-- Client Lua Script for MinimalistResourceBars
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------

require "Window"
require "GameLib"
require "Unit"

local MinimalistResourceBars = {}

local userSettings = {

	--General
	"setShowFocus", "setColorChange", "setColorChange2",
	"setHideOOC", "setHideStatus", "setShowMarkers",
	
	--Colors
	"setColorResourceBackground", "setColorResourceBorder", "setColorResource",
	"setColorIndicator1", "setColorIndicator2", "setColorFocusBackground",
	"setColorFocusBorder", "setColorFocus",
	
	--Opacty
	"setOpacityResourceBackground", "setOpacityResourceBorder", "setOpacityResourceBar",
	"setOpacityFocusBackground", "setOpacityFocusBorder", "setOpacityFocusBar",
	
	--Sizes
	"setWidth", "setHeight", "setFocusWidth",
	"setFocusHeight",
	
	--Fonts
	"setFont",
}

local defaultSettings = {
	
	--Offsets
	setAnchorOffsets = { 0, 0, 360, 30 }, setFocusAnchorOffsets = { 0, 29, 360, 49 },
	
	--General
	setShowFocus = true, setColorChange = true, setColorChange2 = false,
	setHideOOC = false, setHideStatus = false, setShowMarkers = true,
	
	--Colors
	setOpacity = "FF", setColorResourceBackground = "000000", setColorResourceBorder = "000000",
	setColorFocusBackground = "000000", setColorFocusBorder = "000000", setColorFocus = "FBE9A1",
	
	--Opacity
	setOpacityResourceBackground = 0.5, setOpacityResourceBorder = 1, setOpacityResourceBar = 0.75,
	setOpacityFocusBackground = 0.5, setOpacityFocusBorder = 1, setOpacityFocusBar = 0.75,
	
	--Sizes
	setWidth = 360, setHeight = 30, setFocusWidth = 360,
	setFocusHeight = 20,
	
	--Fonts
	setFont = "Nameplates",
}

local classColors = {
	Engineer = { resource = "31F9F3", indicator1 = "FFC758" },
	Esper = { resource = "AE56D5", indicator1 = "26D1E0" },
	Medic = { resource = "106E37", indicator1 = "6AD271" },
	Spellslinger = { resource = "FFC758", indicator1 = "E53805" },
	Stalker = { resource = "C20FB2", indicator1 = "37D5F7", indicator2 = "7405E5" },
	Warrior = { resource = "E53805", indicator1 = "EDD582", indicator2 = "7405E5" },
}

function MinimalistResourceBars:Init()
	Apollo.RegisterAddon(self, true, "MRB")
end

function MinimalistResourceBars:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("MinimalistResourceBars.xml")
	self.xmlDoc:RegisterCallback("OnDocumentReady", self)
end

function MinimalistResourceBars:OnDocumentReady()
	if self.xmlDoc == nil then
		return
	end
	
	self:OnRequiredFlagsChanged()
end

function MinimalistResourceBars:OnRequiredFlagsChanged()
	if GameLib.GetPlayerUnit() then
		self:OnCharacterCreated()
	else
		Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)
	end
end

function MinimalistResourceBars:OnCharacterCreated()
	
	--Load sprites
	Apollo.LoadSprites("MinimalistResourceBars_Sprites.xml")
	
	--Register slash commands
	Apollo.RegisterSlashCommand("mrb", "OnMinimalistResourceBarsOn", self)

	--Register forms
	self.wndOptions = Apollo.LoadForm(self.xmlDoc, "Options", nil, self)
	self.wndColors = Apollo.LoadForm(self.xmlDoc, "Colors", self.wndOptions:FindChild("Content_Colors"), self)
	self.wndStyle = Apollo.LoadForm(self.xmlDoc, "Style", self.wndOptions:FindChild("Content_Style"), self)
	self.wndFocusBar = Apollo.LoadForm(self.xmlDoc, "ResourceBar", nil, self)
	
	self.wndOptions:Show(false)
	self.wndFocusBar:Show(false)
	
	self.wndOptions:FindChild("Button_General"):SetCheck(true)
	self.wndOptions:FindChild("Content_General"):Show(true)
	self.wndOptions:FindChild("Content_Colors"):Show(false)
	self.wndOptions:FindChild("Content_Style"):Show(false)
	
	--Class check
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer == nil then
		return
	end
	
	local eClassId = unitPlayer:GetClassId()
	if eClassId == GameLib.CodeEnumClass.Engineer then
		self.playerClass = "Engineer"
		self:OnCreatedEngineer()
	elseif eClassId == GameLib.CodeEnumClass.Esper then
		self.playerClass = "Esper"
		self:OnCreatedEsper()
	elseif eClassId == GameLib.CodeEnumClass.Medic then
		self.playerClass = "Medic"
		self:OnCreatedMedic()
	elseif eClassId == GameLib.CodeEnumClass.Spellslinger then
		self.playerClass = "Spellslinger"
		self:OnCreatedSpellslinger()
	elseif eClassId == GameLib.CodeEnumClass.Stalker then
		self.playerClass = "Stalker"
		self:OnCreatedStalker()
	elseif eClassId == GameLib.CodeEnumClass.Warrior then
		self.playerClass = "Warrior"
		self:OnCreatedWarrior()
	end
	
	--Load settings
	self:DefaultSettings()
	self:RefreshSettings()
	
	--Update resource bar
	self:SetPosition()
	self:SetColors()
	self:SetFont()
	self:SetMarkers()
end

function MinimalistResourceBars:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end

	local tSavedData = {}
	tSavedData.setAnchorOffsets = { self.wndResourceBar:GetAnchorOffsets() }
	tSavedData.setFocusAnchorOffsets = { self.wndFocusBar:GetAnchorOffsets() }
	
	for _,v in ipairs(userSettings) do tSavedData[v] = self[v] end
	
	return tSavedData
end

function MinimalistResourceBars:OnRestore(eType, tSavedData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	
	if tSavedData.setAnchorOffsets then
		self.setAnchorOffsets = tSavedData.setAnchorOffsets
	end
	
	if tSavedData.setFocusAnchorOffsets then
		self.setFocusAnchorOffsets = tSavedData.setFocusAnchorOffsets
	end
	
	for _,v in ipairs(userSettings) do
		if tSavedData[v] ~= nil then self[v] = tSavedData[v] end
	end
end

function MinimalistResourceBars:OnMinimalistResourceBarsOn()
	self.wndOptions:ToFront()
	self.wndOptions:Show(true)
	self:RefreshSettings()
end

function MinimalistResourceBars:OnConfigure()
	if self.wndOptions ~= nil then
		self:OnMinimalistResourceBarsOn()
	end
end

function MinimalistResourceBars:DefaultSettings()
	
	--Colors
	if self.setColorResource == nil then
		self.setColorResource = classColors[self.playerClass].resource
	end
	
	if self.setColorIndicator1 == nil then
		self.setColorIndicator1 = classColors[self.playerClass].indicator1
	end
	
	if self.setColorIndicator2 == nil then
		if classColors[self.playerClass].indicator2 ~= nil then
			self.setColorIndicator2 = classColors[self.playerClass].indicator2
		end
	end
	
	--Defaults
	for i,v in pairs(defaultSettings) do
		if self[i] == nil then
			self[i] = v
		end
	end
end

function MinimalistResourceBars:RefreshSettings()
	
	--General
	if self.setColorChange ~= nil then
		self.wndGeneral:FindChild("Button_ColorChange"):SetCheck(self.setColorChange) end
	if self.setHideOOC ~= nil then
		self.wndGeneral:FindChild("Button_HideOOC"):SetCheck(self.setHideOOC) end
	if self.setHideStatus ~= nil then
		self.wndGeneral:FindChild("Button_HideStatus"):SetCheck(self.setHideStatus) end
	
	--Colors
	if self.setColorResourceBackground ~= nil then
		self.wndColors:FindChild("Edit_ResourceBackground"):SetText(self.setColorResourceBackground)
		self.wndColors:FindChild("Edit_ResourceBackground"):SetTextColor(self.setOpacity..self.setColorResourceBackground) end
	if self.setColorResourceBorder ~= nil then
		self.wndColors:FindChild("Edit_ResourceBorder"):SetText(self.setColorResourceBorder)
		self.wndColors:FindChild("Edit_ResourceBorder"):SetTextColor(self.setOpacity..self.setColorResourceBorder) end
	if self.setColorResource ~= nil then
		self.wndColors:FindChild("Edit_Resource"):SetText(self.setColorResource)
		self.wndColors:FindChild("Edit_Resource"):SetTextColor(self.setOpacity..self.setColorResource) end
	if self.setColorIndicator1 ~= nil then
		self.wndColors:FindChild("Edit_Indicator1"):SetText(self.setColorIndicator1)
		self.wndColors:FindChild("Edit_Indicator1"):SetTextColor(self.setOpacity..self.setColorIndicator1) end
	if self.setColorIndicator2 ~= nil then
		self.wndColors:FindChild("Edit_Indicator2"):SetText(self.setColorIndicator2)
		self.wndColors:FindChild("Edit_Indicator2"):SetTextColor(self.setOpacity..self.setColorIndicator2) else
		self.wndColors:FindChild("Indicator2"):Show(false) end
	if self.setColorFocusBackground ~= nil then
		self.wndColors:FindChild("Edit_FocusBackground"):SetText(self.setColorFocusBackground)
		self.wndColors:FindChild("Edit_FocusBackground"):SetTextColor(self.setOpacity..self.setColorFocusBackground) end
	if self.setColorFocusBorder ~= nil then
		self.wndColors:FindChild("Edit_FocusBorder"):SetText(self.setColorFocusBorder)
		self.wndColors:FindChild("Edit_FocusBorder"):SetTextColor(self.setOpacity..self.setColorFocusBorder) end
	if self.setColorFocus ~= nil then
		self.wndColors:FindChild("Edit_Focus"):SetText(self.setColorFocus)
		self.wndColors:FindChild("Edit_Focus"):SetTextColor(self.setOpacity..self.setColorFocus) end
		
	--Opacity
	if self.setOpacityResourceBackground ~= nil then
		self.wndColors:FindChild("Label_ResourceBackground"):SetText("Resource Background ("..(self.setOpacityResourceBackground * 100).."%)")
		self.wndColors:FindChild("Slider_ResourceBackground"):SetValue(self.setOpacityResourceBackground * 100) end
	if self.setOpacityResourceBorder ~= nil then
		self.wndColors:FindChild("Label_ResourceBorder"):SetText("Resource Border ("..(self.setOpacityResourceBorder * 100).."%)")
		self.wndColors:FindChild("Slider_ResourceBorder"):SetValue(self.setOpacityResourceBorder * 100) end
	if self.setOpacityResourceBar ~= nil then
		self.wndColors:FindChild("Label_ResourceBar"):SetText("Resource Bar ("..(self.setOpacityResourceBar * 100).."%)")
		self.wndColors:FindChild("Slider_ResourceBar"):SetValue(self.setOpacityResourceBar * 100) end
	if self.setOpacityFocusBackground ~= nil then
		self.wndColors:FindChild("Label_FocusBackground"):SetText("Focus Background ("..(self.setOpacityFocusBackground * 100).."%)")
		self.wndColors:FindChild("Slider_FocusBackground"):SetValue(self.setOpacityFocusBackground * 100) end
	if self.setOpacityFocusBorder ~= nil then
		self.wndColors:FindChild("Label_FocusBorder"):SetText("Focus Border ("..(self.setOpacityFocusBorder * 100).."%)")
		self.wndColors:FindChild("Slider_FocusBorder"):SetValue(self.setOpacityFocusBorder * 100) end
	if self.setOpacityFocusBar ~= nil then
		self.wndColors:FindChild("Label_FocusBar"):SetText("Focus Bar ("..(self.setOpacityFocusBar * 100).."%)")
		self.wndColors:FindChild("Slider_FocusBar"):SetValue(self.setOpacityFocusBar * 100) end
	
	--Sizes
	if self.setWidth ~= nil then
		self.wndStyle:FindChild("Label_Width"):SetText("Width ("..self.setWidth..")")
		self.wndStyle:FindChild("Slider_Width"):SetValue(self.setWidth) end
	if self.setHeight ~= nil then
		self.wndStyle:FindChild("Label_Height"):SetText("Height ("..self.setHeight..")")
		self.wndStyle:FindChild("Slider_Height"):SetValue(self.setHeight) end
	if self.setFocusWidth ~= nil then
		self.wndStyle:FindChild("Label_FocusWidth"):SetText("Width ("..self.setFocusWidth..")")
		self.wndStyle:FindChild("Slider_FocusWidth"):SetValue(self.setFocusWidth) end
	if self.setFocusHeight ~= nil then
		self.wndStyle:FindChild("Label_FocusHeight"):SetText("Height ("..self.setFocusHeight..")")
		self.wndStyle:FindChild("Slider_FocusHeight"):SetValue(self.setFocusHeight) end
		
	--Font
	if self.setFont ~= nil then
		self.wndStyle:FindChild("Button_SetFontDefault"):SetCheck(self.setFont == "Nameplates")
		self.wndStyle:FindChild("Button_SetFontInterface_Small"):SetCheck(self.setFont == "CRB_InterfaceSmall_O")
		self.wndStyle:FindChild("Button_SetFontInterface_Medium"):SetCheck(self.setFont == "CRB_InterfaceMedium_O")
		self.wndStyle:FindChild("Button_SetFontInterface_Large"):SetCheck(self.setFont == "CRB_InterfaceLarge_O")
		self.wndStyle:FindChild("Button_SetFontPixel"):SetCheck(self.setFont == "CRB_Pixel_O")
		self.wndStyle:FindChild("Button_SetFontSubtitle"):SetCheck(self.setFont == "Subtitle")
		self.wndStyle:FindChild("Button_SetFontThick"):SetCheck(self.setFont == "Thick")
		self.wndStyle:FindChild("Button_SetFontFloat"):SetCheck(self.setFont == "CRB_FloaterMedium") end
		
	--Focus
	if self.playerClass == "Esper" or self.playerClass == "Medic" or self.playerClass == "Spellslinger" then
		if self.setShowFocus ~= nil then
			self.wndGeneral:FindChild("Button_ShowFocus"):SetCheck(self.setShowFocus) end end
			
	--Engineer
	if self.playerClass == "Engineer" then
		if self.setShowMarkers ~= nil then
			self.wndGeneral:FindChild("Button_ShowMarkers"):SetCheck(self.setShowMarkers) end end
			
	--Stalker
	if self.playerClass == "Stalker" then
		if self.setColorChange2 then
			self.wndGeneral:FindChild("Button_ColorChange2"):SetCheck(self.setColorChange2) end end
			
	--Warrior
	if self.playerClass == "Warrior" then
		if self.setColorChange2 then
			self.wndGeneral:FindChild("Button_ColorChange2"):SetCheck(self.setColorChange2) end end
end

function MinimalistResourceBars:SetPosition()
	self.wndResourceBar:SetAnchorOffsets(unpack(self.setAnchorOffsets))
	self.setAnchorOffsets = { self.wndResourceBar:GetAnchorOffsets() }
	
	self.wndFocusBar:SetAnchorOffsets(unpack(self.setFocusAnchorOffsets))
	self.setFocusAnchorOffsets = { self.wndFocusBar:GetAnchorOffsets() }
end

function MinimalistResourceBars:SetColors()
	self.wndResourceBar:FindChild("Background"):SetBGColor(self:ConvertAlpha(self.setOpacityResourceBackground)..self.setColorResourceBackground)
	self.wndResourceBar:FindChild("Border"):SetBGColor(self:ConvertAlpha(self.setOpacityResourceBorder)..self.setColorResourceBorder)
	
	for i=1,5 do
		if self.wndResourceBar:FindChild("Node"..i) ~= nil then
			self.wndResourceBar:FindChild("Node"..i..":Border"..i):SetBGColor(self:ConvertAlpha(self.setOpacityResourceBorder)..self.setColorResourceBorder)
		end
	end
	
	self.wndFocusBar:FindChild("Background"):SetBGColor(self:ConvertAlpha(self.setOpacityFocusBackground)..self.setColorFocusBackground)
	self.wndFocusBar:FindChild("Border"):SetBGColor(self:ConvertAlpha(self.setOpacityFocusBorder)..self.setColorFocusBorder)
end

function MinimalistResourceBars:SetFont()
	self.wndResourceBar:FindChild("ProgressText"):SetFont(self.setFont)
	self.wndFocusBar:FindChild("ProgressText"):SetFont(self.setFont)
end

function MinimalistResourceBars:SetMarkers()
	if self.playerClass == "Engineer" then
		self.wndResourceBar:FindChild("Markers"):Show(self.setShowMarkers)
	end
end

function MinimalistResourceBars:ConvertAlpha(value)	
	--converts a value between 0 and 1 to a hex string
	return string.format("%02X", math.floor(value * 255 + 0.5))
end

-----------------------------------------------------------------------------------------------
-- Engineer
-----------------------------------------------------------------------------------------------
function MinimalistResourceBars:OnCreatedEngineer()

	--Register event handlers
	Apollo.RegisterEventHandler("FrameCount", "OnFrameEngineer", self)

	self.wndGeneral = Apollo.LoadForm(self.xmlDoc, "General_Engineer", self.wndOptions:FindChild("Content_General"), self)
	self.wndResourceBar = Apollo.LoadForm(self.xmlDoc, "ResourceBar", nil, self)
	
	self.wndResourceBar:Show(false)
end

function MinimalistResourceBars:OnFrameEngineer()
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer == nil then
		return
	end
	
	local nResourceCurrent = unitPlayer:GetResource(1)
	local nResourceMax = unitPlayer:GetMaxResource(1)
	local bInCombat = unitPlayer:IsInCombat()
	
	if self.setHideOOC and not bInCombat and (nResourceCurrent == 0 or not self.setHideStatus) then
		if self.wndResourceBar:IsShown() then
			self.wndResourceBar:Show(false)
		end
		return
	end
	
	self.wndResourceBar:FindChild("ProgressBar"):SetMax(nResourceMax)
	self.wndResourceBar:FindChild("ProgressBar"):SetProgress(nResourceCurrent)
	self.wndResourceBar:FindChild("ProgressText"):SetText(nResourceCurrent)
	
	if self.setColorChange and (nResourceCurrent >= 30 and nResourceCurrent <= 70) then
		self.wndResourceBar:FindChild("ProgressBar"):SetBarColor(self:ConvertAlpha(self.setOpacityResourceBar)..self.setColorIndicator1)
	else
		self.wndResourceBar:FindChild("ProgressBar"):SetBarColor(self:ConvertAlpha(self.setOpacityResourceBar)..self.setColorResource)
	end
	
	if not self.wndResourceBar:IsShown() then
		self.wndResourceBar:Show(true)
	end
end

-----------------------------------------------------------------------------------------------
-- Esper
-----------------------------------------------------------------------------------------------
function MinimalistResourceBars:OnCreatedEsper()

	--Register event handlers
	Apollo.RegisterEventHandler("FrameCount", "OnFrameEsper", self)

	self.wndGeneral = Apollo.LoadForm(self.xmlDoc, "General_Esper", self.wndOptions:FindChild("Content_General"), self)
	self.wndResourceBar = Apollo.LoadForm(self.xmlDoc, "ResourceBarNode5", nil, self)

	self.wndResourceBar:Show(false)
	
	for i=1,5 do
		self.wndResourceBar:FindChild("ProgressBar"..i):SetMax(1)
	end
end

function MinimalistResourceBars:OnFrameEsper()
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer == nil then
		return
	end
	
	local nResourceCurrent = unitPlayer:GetResource(1)
	local nResourceMax = unitPlayer:GetMaxResource(1)
	local nManaCurrent = math.floor(unitPlayer:GetFocus())
	local nManaMax = math.floor(unitPlayer:GetMaxFocus())
	local bInCombat = unitPlayer:IsInCombat()
	
	if self.setHideOOC and not bInCombat and (nResourceCurrent == 0 or not self.setHideStatus) then
		if self.wndResourceBar:IsShown() then
			self.wndResourceBar:Show(false)
		end
		if self.wndFocusBar:IsShown() then
			self.wndFocusBar:Show(false)
		end
		return
	end
	
	--Psi Points
	for i=1,5 do
		if nResourceCurrent >= i then
			self.wndResourceBar:FindChild("ProgressBar"..i):SetProgress(1)
		else
			self.wndResourceBar:FindChild("ProgressBar"..i):SetProgress(0)
		end
		if self.setColorChange and nResourceCurrent == 5 then
			self.wndResourceBar:FindChild("ProgressBar"..i):SetBarColor(self:ConvertAlpha(self.setOpacityResourceBar)..self.setColorIndicator1)
		else
			self.wndResourceBar:FindChild("ProgressBar"..i):SetBarColor(self:ConvertAlpha(self.setOpacityResourceBar)..self.setColorResource)
		end
	end
	
	if not self.wndResourceBar:IsShown() then
		self.wndResourceBar:Show(true)
	end
	
	--Focus
	if self.setShowFocus then
		self.wndFocusBar:FindChild("ProgressBar"):SetMax(nManaMax)
		self.wndFocusBar:FindChild("ProgressBar"):SetProgress(nManaCurrent)
		self.wndFocusBar:FindChild("ProgressBar"):SetBarColor(self:ConvertAlpha(self.setOpacityFocusBar)..self.setColorFocus)
		self.wndFocusBar:FindChild("ProgressText"):SetText(nManaCurrent)
		
		if not self.wndFocusBar:IsShown() then
			self.wndFocusBar:Show(true)
		end
	else
		if self.wndFocusBar:IsShown() then
			self.wndFocusBar:Show(false)
		end
	end
end

-----------------------------------------------------------------------------------------------
-- Medic
-----------------------------------------------------------------------------------------------
function MinimalistResourceBars:OnCreatedMedic()

	--Register event handlers
	Apollo.RegisterEventHandler("FrameCount", "OnFrameMedic", self)

	self.wndGeneral = Apollo.LoadForm(self.xmlDoc, "General_Medic", self.wndOptions:FindChild("Content_General"), self)
	self.wndResourceBar = Apollo.LoadForm(self.xmlDoc, "ResourceBarNode4", nil, self)
	
	self.wndResourceBar:Show(false)
	
	for i=1,4 do
		self.wndResourceBar:FindChild("ProgressBar"..i):Show(false)
	end
end

function MinimalistResourceBars:OnFrameMedic()
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer == nil then
		return
	end
	
	local nResourceCurrent = unitPlayer:GetResource(1)
	local nResourceMax = unitPlayer:GetMaxResource(1)
	local nManaCurrent = math.floor(unitPlayer:GetFocus())
	local nManaMax = math.floor(unitPlayer:GetMaxFocus())
	local bInCombat = unitPlayer:IsInCombat()
	local tBuffs = unitPlayer:GetBuffs()
	
	if self.setHideOOC and not bInCombat and (nResourceCurrent == nResourceMax or not self.setHideStatus) then
		if self.wndResourceBar:IsShown() then
			self.wndResourceBar:Show(false)
		end
		if self.wndFocusBar:IsShown() then
			self.wndFocusBar:Show(false)
		end
		return
	end
	
	--Actuators
	local nPartialCount = 0
	for idx, tCurrBuffData in pairs(tBuffs.arBeneficial or {}) do
		if tCurrBuffData.splEffect:GetId() == 42569 then
			nPartialCount = tCurrBuffData.nCount
			break
		end
	end
		
	for i=1,4 do
		local bFull = nResourceCurrent >= i
		local bShowPartial = (nResourceCurrent + 1) == i
		local bPartial1 = bFull or (bShowPartial and nPartialCount >= 1)
		local bPartial2 = bFull or (bShowPartial and nPartialCount >= 2)
		local tempOpacity = self:ConvertAlpha(self.setOpacityResourceBar)
		
		if not bFull then
			tempOpacity = self:ConvertAlpha(self.setOpacityResourceBar / 2)
		end
		
		self.wndResourceBar:FindChild("Node"..i..":Medic:Partial1"):Show(bPartial1, not bPartial1)
		self.wndResourceBar:FindChild("Node"..i..":Medic:Partial2"):Show(bPartial2, not bPartial2)
		self.wndResourceBar:FindChild("Node"..i..":Medic:Partial3"):Show(bFull, not bFull)
		
		for k=1,3 do
			if self.setColorChange and nResourceCurrent == 4 then
				self.wndResourceBar:FindChild("Node"..i..":Medic:Partial"..k):SetBGColor(tempOpacity..self.setColorIndicator1)
			else
				self.wndResourceBar:FindChild("Node"..i..":Medic:Partial"..k):SetBGColor(tempOpacity..self.setColorResource)
			end
		end
	end
	
	if not self.wndResourceBar:IsShown() then
		self.wndResourceBar:Show(true)
	end
	
	--Focus
	if self.setShowFocus then
		self.wndFocusBar:FindChild("ProgressBar"):SetMax(nManaMax)
		self.wndFocusBar:FindChild("ProgressBar"):SetProgress(nManaCurrent)
		self.wndFocusBar:FindChild("ProgressBar"):SetBarColor(self:ConvertAlpha(self.setOpacityFocusBar)..self.setColorFocus)
		self.wndFocusBar:FindChild("ProgressText"):SetText(nManaCurrent)
		
		if not self.wndFocusBar:IsShown() then
			self.wndFocusBar:Show(true)
		end
	else
		if self.wndFocusBar:IsShown() then
			self.wndFocusBar:Show(false)
		end
	end
end

-----------------------------------------------------------------------------------------------
-- Spellslinger
-----------------------------------------------------------------------------------------------
function MinimalistResourceBars:OnCreatedSpellslinger()

	--Register event handlers
	Apollo.RegisterEventHandler("FrameCount", "OnFrameSpellslinger", self)

	self.wndGeneral = Apollo.LoadForm(self.xmlDoc, "General_Spellslinger", self.wndOptions:FindChild("Content_General"), self)
	self.wndResourceBar = Apollo.LoadForm(self.xmlDoc, "ResourceBarNode4", nil, self)
	
	self.wndResourceBar:Show(false)
end

function MinimalistResourceBars:OnFrameSpellslinger()
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer == nil then
		return
	end
	
	local nResourceCurrent = unitPlayer:GetResource(4)
	local nResourceMax = unitPlayer:GetMaxResource(4)
	local nResourceMaxDiv4 = nResourceMax / 4
	local nManaCurrent = math.floor(unitPlayer:GetFocus())
	local nManaMax = math.floor(unitPlayer:GetMaxFocus())
	local bSurgeActive = GameLib.IsSpellSurgeActive()
	local bInCombat = unitPlayer:IsInCombat()
	
	if self.setHideOOC and not bInCombat and (nResourceCurrent == nResourceMax or not self.setHideStatus) then
		if self.wndResourceBar:IsShown() then
			self.wndResourceBar:Show(false)
		end
		if self.wndFocusBar:IsShown() then
			self.wndFocusBar:Show(false)
		end
		return
	end
	
	--Spell Power
	for i=1,4 do
		local nPartialProgress = nResourceCurrent - (nResourceMaxDiv4 * (i - 1))
		local bThisBubbleFilled = nPartialProgress >= nResourceMaxDiv4
		local tempOpacity = self:ConvertAlpha(self.setOpacityResourceBar)
		
		if nResourceCurrent < (nResourceMaxDiv4 * i) then
			tempOpacity = self:ConvertAlpha(self.setOpacityResourceBar / 2)
		end
		
		self.wndResourceBar:FindChild("ProgressBar"..i):SetMax(nResourceMaxDiv4)
		self.wndResourceBar:FindChild("ProgressBar"..i):SetProgress(nPartialProgress, 100)
		
		if self.setColorChange and bSurgeActive then
			self.wndResourceBar:FindChild("ProgressBar"..i):SetBarColor(tempOpacity..self.setColorIndicator1)
		else
			self.wndResourceBar:FindChild("ProgressBar"..i):SetBarColor(tempOpacity..self.setColorResource)
		end
	end
	
	if not self.wndResourceBar:IsShown() then
		self.wndResourceBar:Show(true)
	end
	
	--Focus
	if self.setShowFocus then
		self.wndFocusBar:FindChild("ProgressBar"):SetMax(nManaMax)
		self.wndFocusBar:FindChild("ProgressBar"):SetProgress(nManaCurrent)
		self.wndFocusBar:FindChild("ProgressBar"):SetBarColor(self:ConvertAlpha(self.setOpacityFocusBar)..self.setColorFocus)
		self.wndFocusBar:FindChild("ProgressText"):SetText(nManaCurrent)
		
		if not self.wndFocusBar:IsShown() then
			self.wndFocusBar:Show(true)
		end
	else
		if self.wndFocusBar:IsShown() then
			self.wndFocusBar:Show(false)
		end
	end
end

-----------------------------------------------------------------------------------------------
-- Stalker
-----------------------------------------------------------------------------------------------
function MinimalistResourceBars:OnCreatedStalker()

	--Register event handlers
	Apollo.RegisterEventHandler("FrameCount",	"OnFrameStalker", self)
	
	--Register timer
	Apollo.RegisterTimerHandler("TimerStalker", "OnTimerStalker", self)
	Apollo.CreateTimer("TimerStalker", 0.1, true)

	self.wndGeneral = Apollo.LoadForm(self.xmlDoc, "General_Stalker", self.wndOptions:FindChild("Content_General"), self)
	self.wndResourceBar = Apollo.LoadForm(self.xmlDoc, "ResourceBar", nil, self)
	
	self.wndResourceBar:Show(false)
end

function MinimalistResourceBars:OnFrameStalker()
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer == nil then
		return
	end
	
	local nResourceCurrent = unitPlayer:GetResource(3)
	local nResourceMax = unitPlayer:GetMaxResource(3)
	local bInCombat = unitPlayer:IsInCombat()
	
	if self.setHideOOC and not bInCombat and (nResourceCurrent == nResourceMax or not self.setHideStatus) then
		if self.wndResourceBar:IsShown() then
			self.wndResourceBar:Show(false)
		end
		return
	end
	
	self.wndResourceBar:FindChild("ProgressBar"):SetMax(nResourceMax)
	self.wndResourceBar:FindChild("ProgressBar"):SetProgress(nResourceCurrent)
	self.wndResourceBar:FindChild("ProgressText"):SetText(nResourceCurrent)
	
	if self.setColorChange2 and self.bStealthActive then
		self.wndResourceBar:FindChild("ProgressBar"):SetBarColor(self:ConvertAlpha(self.setOpacityResourceBar)..self.setColorIndicator2)
	elseif self.setColorChange and nResourceCurrent < 35 then
		self.wndResourceBar:FindChild("ProgressBar"):SetBarColor(self:ConvertAlpha(self.setOpacityResourceBar)..self.setColorIndicator1)
	else
		self.wndResourceBar:FindChild("ProgressBar"):SetBarColor(self:ConvertAlpha(self.setOpacityResourceBar)..self.setColorResource)
	end
	
	if not self.wndResourceBar:IsShown() then
		self.wndResourceBar:Show(true)
	end
end

function MinimalistResourceBars:OnTimerStalker()
	local unitPlayer = GameLib.GetPlayerUnit()
	
	if unitPlayer ~= nil and unitPlayer:IsValid() then
		local buffs = unitPlayer:GetBuffs()
		
		if buffs ~= nil then
			for _,v in ipairs(buffs.arBeneficial) do
				if v.splEffect:GetId() == 38784 then
					self.bStealthActive = true
					return
				end
			end
		end
	end
	self.bStealthActive = false
end

-----------------------------------------------------------------------------------------------
-- Warrior
-----------------------------------------------------------------------------------------------
function MinimalistResourceBars:OnCreatedWarrior()

	--Register event handlers
	Apollo.RegisterEventHandler("FrameCount", "OnFrameWarrior", self)

	self.wndGeneral = Apollo.LoadForm(self.xmlDoc, "General_Warrior", self.wndOptions:FindChild("Content_General"), self)
	self.wndResourceBar = Apollo.LoadForm(self.xmlDoc, "ResourceBar", nil, self)
	
	self.wndResourceBar:Show(false)
end

function MinimalistResourceBars:OnFrameWarrior()
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer == nil then
		return
	end
	
	local nResourceCurrent = unitPlayer:GetResource(1)
	local nResourceMax = unitPlayer:GetMaxResource(1)
	local bInCombat = unitPlayer:IsInCombat()
	
	if self.setHideOOC and not bInCombat and (nResourceCurrent == 0 or not self.setHideStatus) then
		if self.wndResourceBar:IsShown() then
			self.wndResourceBar:Show(false)
		end
		return
	end
	
	self.wndResourceBar:FindChild("ProgressBar"):SetMax(nResourceMax)
	self.wndResourceBar:FindChild("ProgressBar"):SetProgress(nResourceCurrent)
	self.wndResourceBar:FindChild("ProgressText"):SetText(nResourceCurrent)
	
	if self.setColorChange and nResourceCurrent >= 750 then
		self.wndResourceBar:FindChild("ProgressBar"):SetBarColor(self:ConvertAlpha(self.setOpacityResourceBar)..self.setColorIndicator1)
	elseif self.setColorChange2 and nResourceCurrent < 500 then
		self.wndResourceBar:FindChild("ProgressBar"):SetBarColor(self:ConvertAlpha(self.setOpacityResourceBar)..self.setColorIndicator2)
	else
		self.wndResourceBar:FindChild("ProgressBar"):SetBarColor(self:ConvertAlpha(self.setOpacityResourceBar)..self.setColorResource)
	end
	
	if not self.wndResourceBar:IsShown() then
		self.wndResourceBar:Show(true)
	end
end

-----------------------------------------------------------------------------------------------
-- Buttons
-----------------------------------------------------------------------------------------------
function MinimalistResourceBars:Button_General(wndHandler, wndControl)
	self.wndOptions:FindChild("Content_General"):Show(true)
	self.wndOptions:FindChild("Content_Colors"):Show(false)
	self.wndOptions:FindChild("Content_Style"):Show(false)
	
	self.wndOptions:FindChild("Content_General"):SetVScrollPos(0)
end

function MinimalistResourceBars:Button_Colors(wndHandler, wndControl)
	self.wndOptions:FindChild("Content_General"):Show(false)
	self.wndOptions:FindChild("Content_Colors"):Show(true)
	self.wndOptions:FindChild("Content_Style"):Show(false)
	
	self.wndOptions:FindChild("Content_Colors"):SetVScrollPos(0)
end

function MinimalistResourceBars:Button_Style(wndHandler, wndControl)
	self.wndOptions:FindChild("Content_General"):Show(false)
	self.wndOptions:FindChild("Content_Colors"):Show(false)
	self.wndOptions:FindChild("Content_Style"):Show(true)
	
	self.wndOptions:FindChild("Content_Style"):SetVScrollPos(0)
end

function MinimalistResourceBars:Button_Close()
	self.wndOptions:Show(false)
end

function MinimalistResourceBars:Button_ShowFocus(wndHandler, wndControl)
	self.setShowFocus = wndControl:IsChecked()
end

function MinimalistResourceBars:Button_ColorChange(wndHandler, wndControl)
	self.setColorChange = wndControl:IsChecked()
end

function MinimalistResourceBars:Button_ColorChange2(wndHandler, wndControl)
	self.setColorChange2 = wndControl:IsChecked()
end

function MinimalistResourceBars:Button_HideOOC(wndHandler, wndControl)
	self.setHideOOC = wndControl:IsChecked()
end

function MinimalistResourceBars:Button_HideStatus(wndHandler, wndControl)
	self.setHideStatus = wndControl:IsChecked()
end

function MinimalistResourceBars:Button_ShowMarkers(wndHandler, wndControl)
	self.setShowMarkers = wndControl:IsChecked()
	self:SetMarkers()
end

function MinimalistResourceBars:Button_SetFontDefault()
	self.setFont = "Nameplates"
	self:SetFont()
end

function MinimalistResourceBars:Button_SetFontInterface_Small()
	self.setFont = "CRB_InterfaceSmall_O"
	self:SetFont()
end

function MinimalistResourceBars:Button_SetFontInterface_Medium()
	self.setFont = "CRB_InterfaceMedium_O"
	self:SetFont()
end

function MinimalistResourceBars:Button_SetFontInterface_Large()
	self.setFont = "CRB_InterfaceLarge_O"
	self:SetFont()
end

function MinimalistResourceBars:Button_SetFontPixel()
	self.setFont = "CRB_Pixel_O"
	self:SetFont()
end

function MinimalistResourceBars:Button_SetFontSubtitle()
	self.setFont = "Subtitle"
	self:SetFont()
end

function MinimalistResourceBars:Button_SetFontThick()
	self.setFont = "Thick"
	self:SetFont()
end

function MinimalistResourceBars:Button_SetFontFloat()
	self.setFont = "CRB_FloaterMedium"
	self:SetFont()
end

-----------------------------------------------------------------------------------------------
-- EditBoxes
-----------------------------------------------------------------------------------------------
function MinimalistResourceBars:Edit_ResourceBackground()
	local colorString = self.wndColors:FindChild("Edit_ResourceBackground"):GetText()
	
	if string.len(colorString) > 6 then
		self.wndColors:FindChild("Edit_ResourceBackground"):SetText(string.sub(colorString, 0, 6))
	end
	
	if string.len(colorString) == 6 then
		self.wndColors:FindChild("Edit_ResourceBackground"):SetTextColor(self.setOpacity..colorString)
		self.setColorResourceBackground = colorString
		self:SetColors()
	end
end

function MinimalistResourceBars:Edit_ResourceBorder()
	local colorString = self.wndColors:FindChild("Edit_ResourceBorder"):GetText()
	
	if string.len(colorString) > 6 then
		self.wndColors:FindChild("Edit_ResourceBorder"):SetText(string.sub(colorString, 0, 6))
	end
	
	if string.len(colorString) == 6 then
		self.wndColors:FindChild("Edit_ResourceBorder"):SetTextColor(self.setOpacity..colorString)
		self.setColorResourceBorder = colorString
		self:SetColors()
	end
end

function MinimalistResourceBars:Edit_Resource()
	local colorString = self.wndColors:FindChild("Edit_Resource"):GetText()
	
	if string.len(colorString) > 6 then
		self.wndColors:FindChild("Edit_Resource"):SetText(string.sub(colorString, 0, 6))
	end
	
	if string.len(colorString) == 6 then
		self.wndColors:FindChild("Edit_Resource"):SetTextColor(self.setOpacity..colorString)
		self.setColorResource = colorString
	end
end

function MinimalistResourceBars:Edit_Indicator1()
	local colorString = self.wndColors:FindChild("Edit_Indicator1"):GetText()
	
	if string.len(colorString) > 6 then
		self.wndColors:FindChild("Edit_Indicator1"):SetText(string.sub(colorString, 0, 6))
	end
	
	if string.len(colorString) == 6 then
		self.wndColors:FindChild("Edit_Indicator1"):SetTextColor(self.setOpacity..colorString)
		self.setColorIndicator1 = colorString
	end
end

function MinimalistResourceBars:Edit_Indicator2()
	local colorString = self.wndColors:FindChild("Edit_Indicator2"):GetText()
	
	if string.len(colorString) > 6 then
		self.wndColors:FindChild("Edit_Indicator2"):SetText(string.sub(colorString, 0, 6))
	end
	
	if string.len(colorString) == 6 then
		self.wndColors:FindChild("Edit_Indicator2"):SetTextColor(self.setOpacity..colorString)
		self.setColorIndicator2 = colorString
	end
end

function MinimalistResourceBars:Edit_FocusBackground()
	local colorString = self.wndColors:FindChild("Edit_FocusBackground"):GetText()
	
	if string.len(colorString) > 6 then
		self.wndColors:FindChild("Edit_FocusBackground"):SetText(string.sub(colorString, 0, 6))
	end
	
	if string.len(colorString) == 6 then
		self.wndColors:FindChild("Edit_FocusBackground"):SetTextColor(self.setOpacity..colorString)
		self.setColorFocusBackground = colorString
		self:SetColors()
	end
end

function MinimalistResourceBars:Edit_FocusBorder()
	local colorString = self.wndColors:FindChild("Edit_FocusBorder"):GetText()
	
	if string.len(colorString) > 6 then
		self.wndColors:FindChild("Edit_FocusBorder"):SetText(string.sub(colorString, 0, 6))
	end
	
	if string.len(colorString) == 6 then
		self.wndColors:FindChild("Edit_FocusBorder"):SetTextColor(self.setOpacity..colorString)
		self.setColorFocusBorder = colorString
		self:SetColors()
	end
end

function MinimalistResourceBars:Edit_Focus()
	local colorString = self.wndColors:FindChild("Edit_Focus"):GetText()
	
	if string.len(colorString) > 6 then
		self.wndColors:FindChild("Edit_Focus"):SetText(string.sub(colorString, 0, 6))
	end
	
	if string.len(colorString) == 6 then
		self.wndColors:FindChild("Edit_Focus"):SetTextColor(self.setOpacity..colorString)
		self.setColorFocus = colorString
	end
end

-----------------------------------------------------------------------------------------------
-- Sliders
-----------------------------------------------------------------------------------------------
function MinimalistResourceBars:Slider_ResourceBackground(wndHandler, wndControl, nValue, nOldValue)
	if nValue ~= nOldValue then
		self.setOpacityResourceBackground = nValue / 100
		self.wndColors:FindChild("Label_ResourceBackground"):SetText("Resource Background ("..(nValue).."%)")
		self:SetColors()
	end
end

function MinimalistResourceBars:Slider_ResourceBorder(wndHandler, wndControl, nValue, nOldValue)
	if nValue ~= nOldValue then
		self.setOpacityResourceBorder = nValue / 100
		self.wndColors:FindChild("Label_ResourceBorder"):SetText("Resource Border ("..(nValue).."%)")
		self:SetColors()
	end
end

function MinimalistResourceBars:Slider_ResourceBar(wndHandler, wndControl, nValue, nOldValue)
	if nValue ~= nOldValue then
		self.setOpacityResourceBar = nValue / 100
		self.wndColors:FindChild("Label_ResourceBar"):SetText("Resource Bar ("..(nValue).."%)")
	end
end

function MinimalistResourceBars:Slider_FocusBackground(wndHandler, wndControl, nValue, nOldValue)
	if nValue ~= nOldValue then
		self.setOpacityFocusBackground = nValue / 100
		self.wndColors:FindChild("Label_FocusBackground"):SetText("Focus Background ("..(nValue).."%)")
		self:SetColors()
	end
end

function MinimalistResourceBars:Slider_FocusBorder(wndHandler, wndControl, nValue, nOldValue)
	if nValue ~= nOldValue then
		self.setOpacityFocusBorder = nValue / 100
		self.wndColors:FindChild("Label_FocusBorder"):SetText("Focus Border ("..(nValue).."%)")
		self:SetColors()
	end
end

function MinimalistResourceBars:Slider_FocusBar(wndHandler, wndControl, nValue, nOldValue)
	if nValue ~= nOldValue then
		self.setOpacityFocusBar = nValue / 100
		self.wndColors:FindChild("Label_FocusBar"):SetText("Focus Bar ("..(nValue).."%)")
	end
end

function MinimalistResourceBars:Slider_Width(wndHandler, wndControl, nValue, nOldValue)
	if nValue ~= nOldValue then
		local currentOffsets = { self.wndResourceBar:GetAnchorOffsets() }
		local newOffset = currentOffsets[1] + nValue
		
		self.setAnchorOffsets = { currentOffsets[1], currentOffsets[2], newOffset, currentOffsets[4] }
		self.setWidth = nValue
		
		self.wndStyle:FindChild("Label_Width"):SetText("Width ("..nValue..")")
		self.wndResourceBar:SetAnchorOffsets(unpack(self.setAnchorOffsets))
	end
end

function MinimalistResourceBars:Slider_Height(wndHandler, wndControl, nValue, nOldValue)
	if nValue ~= nOldValue then
		local currentOffsets = { self.wndResourceBar:GetAnchorOffsets() }
		local newOffset = currentOffsets[2] + nValue
		
		self.setAnchorOffsets = { currentOffsets[1], currentOffsets[2], currentOffsets[3], newOffset }
		self.setHeight = nValue
		
		self.wndStyle:FindChild("Label_Height"):SetText("Height ("..nValue..")")
		self.wndResourceBar:SetAnchorOffsets(unpack(self.setAnchorOffsets))
	end
end

function MinimalistResourceBars:Slider_FocusWidth(wndHandler, wndControl, nValue, nOldValue)
	if nValue ~= nOldValue then
		local currentOffsets = { self.wndFocusBar:GetAnchorOffsets() }
		local newOffset = currentOffsets[1] + nValue
		
		self.setFocusAnchorOffsets = { currentOffsets[1], currentOffsets[2], newOffset, currentOffsets[4] }
		self.setFocusWidth = nValue
		
		self.wndStyle:FindChild("Label_FocusWidth"):SetText("Width ("..nValue..")")
		self.wndFocusBar:SetAnchorOffsets(unpack(self.setFocusAnchorOffsets))
	end
end

function MinimalistResourceBars:Slider_FocusHeight(wndHandler, wndControl, nValue, nOldValue)
	if nValue ~= nOldValue then
		local currentOffsets = { self.wndFocusBar:GetAnchorOffsets() }
		local newOffset = currentOffsets[2] + nValue
		
		self.setFocusAnchorOffsets = { currentOffsets[1], currentOffsets[2], currentOffsets[3], newOffset }
		self.setFocusHeight = nValue
		
		self.wndStyle:FindChild("Label_FocusHeight"):SetText("Height ("..nValue..")")
		self.wndFocusBar:SetAnchorOffsets(unpack(self.setFocusAnchorOffsets))
	end
end

MinimalistResourceBars:Init()
